/*
 * $Id: fonts.c 2012-12-30  10:52:34 ssk  $
 *
 * This program is part of TIANMA TM13265LBCWG with 132x65 Graphic LCD Tutorial.
 * Designed for NOVATEK NT7502 65 x 132 RAM-Map LCD Controller/Driver based LCDs on Atmels AVR MCUs
 *
 * Copyright (C) 2012  ssk.
 *
 * Originally Adapted from
 *    Most Functions   : GLCD Version 3 (Beta) Jun 9 2010 <http://arduino.cc/forum/index.php/topic,22208.0.html>
 *    Font Function&Data: Kaiserslautern, Germany <eversmith@heizung-thomas.de> 
 *                       avr-gcc 3.4.3/avr-libc 1.2.3 (WinAVR 2/2005)
 *    Cube Drawing     : Osama's Lab KS0108 GLCD library <https://sites.google.com/site/osamaslab>
 *    Line Drawing     : http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
 *    Circle Drawing   : http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
 *
 *
 * Changed & added additional constants for gcc version 4.3.3 (WinAVR 20100110), improved speed and optimized code
 * You need to be selective about the information you receive instead of accepting it unconditionally.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "sbn166g.h"
#include "fonts.h"

  // Font Indices
#define FONT_LENGTH			   0
#define FONT_FIXED_WIDTH	 2
#define FONT_HEIGHT		   	 3
#define FONT_FIRST_CHAR		 4
#define FONT_CHAR_COUNT		 5
#define FONT_WIDTH_TABLE	 6

uint8_t glcd_readfont(const uint8_t* ptr) 
{
	return pgm_read_byte(ptr);
}

//void glcd_selectfont(const char* font, glcd_font_callback callback, uint8_t color)
void glcd_selectfont(uint8_t* font, glcd_font_callback callback, uint8_t color)
{
	_glcd_font = font;
	_glcd_fontread = callback;
	_glcd_fontcolor = color;
}
/**
 * output a character
 *
 * @param c the character to output
 *
 * If the character will not fit on the current text line
 * inside the text area,
 * the text position is wrapped to the next line.  * 
 *
 */
uint8_t glcd_putc(char c) 
{
  if(_glcd_font == 0) return 1; // no font selected
  if(c < 0x20)        return 1; // special character processing

	uint8_t firstChar = _glcd_fontread(_glcd_font+FONT_FIRST_CHAR);
	uint8_t charCount = _glcd_fontread(_glcd_font+FONT_CHAR_COUNT);
  uint8_t height = _glcd_fontread(_glcd_font+FONT_HEIGHT);
  uint8_t width  = 0;
  uint8_t bytes  = (height+7)/8;
	uint8_t x = _glcd_coord.x;
	uint8_t y = _glcd_coord.y;
  uint8_t data, i,j;
  uint16_t index = 0;


	if(c < firstChar || c >= (firstChar+charCount)) return 1; // invalid char
  c-= firstChar;

  if(isfixedwidthfont(_glcd_font))
  {
		width = _glcd_fontread(_glcd_font+FONT_FIXED_WIDTH); 
		index = c*bytes*width+FONT_WIDTH_TABLE;  	
  }
  else
  {  
	  // variable width font, read width data, to get the index
		/*
		 * Because there is no table for the offset of where the data
		 * for each character glyph starts, run the table and add up all the
		 * widths of all the characters prior to the character we
		 * need to locate.
		 */
 	
	  // read width data, to get the index
	  for(i=0; i<c; i++) index += _glcd_fontread(_glcd_font+FONT_WIDTH_TABLE+i);
	  		/*
	  	 * Calculate the offset of where the font data
	  	 * for our character starts.
	  	 * The index value from above has to be adjusted because
	  	 * there is potentialy more than 1 byte per column in the glyph,
	  	 * when the characgter is taller than 8 bits.
	  	 * To account for this, index has to be multiplied
	  	 * by the height in bytes because there is one byte of font
	  	 * data for each vertical 8 pixels.
	  	 * The index is then adjusted to skip over the font width data
	  	 * and the font header information.
	  	 */
    index = index*bytes+charCount+FONT_WIDTH_TABLE;
     	/*
	  	 * Finally, fetch the width of our character
	  	 */
	  width = _glcd_fontread(_glcd_font+FONT_WIDTH_TABLE+c);
  }
    	
  if((x + width) > LCD_RIGHT) 
  {
  	x = 0;
  	y+= height;
  	glcd_gotoxy(x, y);
  }	
    
	// last but not least, draw the character
  /*
 	 * Draw each column of the glyph (character) horizontally
 	 * 8 bits (1 page) at a time.
 	 * i.e. if a font is taller than 8 bits, draw upper 8 bits first,
 	 * Then drop down and draw next 8 bits and so on, until done.
	 * This code depends on WriteData() doing writes that span LCD
	 * memory pages, which has issues because the font data isn't
	 * always a multiple of 8 bits.
	 */
	
  for( i=0; i<bytes; i++)  /* each vertical byte */
	{
		uint16_t page = i*width;
    for(j=0; j<width; j++) 
		{
			data = _glcd_fontread(_glcd_font+index+page+j);
		 /*
		  * This funkyness is because when the character glyph is not a
		  * multiple of 8 in height, the residual bits in the font data
		  * were aligned to the incorrect end of the byte with respect 
		  * to the GLCD. I believe that this was an initial oversight (bug)
		  * in Thieles font creator program. It is easily fixed
		  * in the font program but then creates a potential backward
		  * compatiblity problem.
		  *	--- bperrybap
		  */
			

			if(height > 8 && height < (i+1)*8) data >>= (i+1)*8-height;   

	    if(_glcd_fontcolor == LCD_DOT_SET) 
     		glcd_data(data);
    	else
				glcd_data(~data);
		}
     // 1px gap between chars
    if(_glcd_fontcolor == LCD_DOT_SET) 
    	glcd_data(0x00);
    else 
    	glcd_data(0xff);
    	
    glcd_gotoxy(x, _glcd_coord.y+8);	
	}
	glcd_gotoxy(x+width+1, y);
	return 0;
}

void glcd_puts(char* str) 
{
	uint8_t x,c;
	
	x = _glcd_coord.x;
  while((c = *str++))
  {
 		if(c == '\n')
  	  glcd_gotoxy(x, _glcd_coord.y +_glcd_fontread(_glcd_font+FONT_HEIGHT));
    else 
     	glcd_putc(c);
	}
}
void glcd_puts_P(PGM_P str)
{
	uint8_t x,c;
	
	x = _glcd_coord.x;
	while((c = pgm_read_byte(str++))) glcd_putc(c);
 		if(c == '\n')
  	  glcd_gotoxy(x, _glcd_coord.y +_glcd_fontread(_glcd_font+FONT_HEIGHT));
    else 
     	glcd_putc(c);
}

/**
 * EOF
*/