/*
 * $Id: main.c 2013/01/29 22:53:05  ssk  $
 *
 * This program is Main of Winstar WG20232A-TMI-V#A with 202x32 Graphic LCD Tutorial.
 *
 * Copyright (C) 2012  ssk.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

/**
 * \mainpage
 * A graphical LCD is an inexpensive, easy-to-control, powerful solution for the display of information. 
 * It can display both text and information independently of an application
 * This project contains the source for a Winstar WG20232A-TMI-V#A with
 * AVANT SBN166G Dot-matrix STN LCD Driver with 32-row x 80-column 
 * 202x32 graphic lcd build around an Atmel ATMEGA324P Microcontroller.
 * Parts of this code might also be used in other projects and with
 * other microcontrollers.
 *
 * Operating voltage range (VDD): 3.0~ 5.5 volts.
 * LCD bias voltage (VLCD=V5-VDD): -13 volts (max.).
 * Operating frequency range: 2 KHz, 18 KHz.
 * Operating temperature range: -15 to +75 C.
 * Storage temperature range: -55 to +125 C.
 * The code is distributed under the terms of the GNU General Public
 * License.
 *
 * \author ssk.
*/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "sbn166g.h"	                      // definitions for the LCD-library
#include "fonts.h"                        // fonts processing functions prototype
#include "fonts/allfonts.h"
#include "fonts/Lorem_Ipsum_Text.h"    // Placeholder text (dummy text)
#include "bitmaps/allbitmaps.h"        // all images in the bitmap dir 
#include "delayNS.h"

#define DELAY 3500     // milliseconds

// Constants in program space        0....+....1....+....
const char pstr_welcome1[] PROGMEM = "Winstar";                // Welcome message display
const char pstr_welcome2[] PROGMEM = "WG20232A-TMI-V#A 202x32";      // Welcome message display
const char pstr_version1[] PROGMEM = "Graphic LCD Tutorial";      // Version number as string
const char pstr_version2[] PROGMEM = "AVANT SBN166G LCD Driver";          // Version number as string

     
//demo functions
static void demo_welcome_image(void);
static void demo_welcome(void);
static void demo_bmp(void);
static void demo_line_graphics(void);
static void demo_font(void);
static void demo_end(void);
static void demo_performace(void);
static void timer1_init(void);
static void drawSpinner(uint8_t pos, uint8_t x, uint8_t y);

volatile uint32_t ISR_milisecond = 0UL;	// milisecond

int main (void)
{
  glcd_init();
  while (1)
  {            
  	glcd_clear(); 
#ifdef PERF_TEST
    demo_performace();
   _delay_ms(DELAY);
    continue;
#endif

    demo_welcome_image();
    demo_welcome();
    demo_bmp();
    demo_line_graphics();
    demo_font();  
    
    demo_end();
  }
   return 0;   /* never reached */
}

static void demo_welcome(void)
{
	glcd_clear();
	
	glcd_bitmap(winstar32x32, 2,0,LCD_DOT_SET);
 	glcd_rect_fill(0,0,34,LCD_BOTTOM,LCD_DOT_XOR);
 	glcd_rect(0,0,LCD_RIGHT,LCD_BOTTOM,LCD_DOT_SET,LCD_LINE_SOLID);

  glcd_selectfont(Arial_Bold_14, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(88, 1); glcd_puts_P(pstr_welcome1);
  
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(48, 14); glcd_puts_P(pstr_welcome2);
  glcd_gotoxy(45,23); glcd_puts_P(pstr_version2);
  glcd_rect_fill(36,13,LCD_RIGHT-2,21,LCD_DOT_XOR);
  _delay_ms(DELAY);

  // invert screen
  glcd_rect_fill(0,0,LCD_RIGHT,LCD_BOTTOM,LCD_DOT_XOR);
  _delay_ms(DELAY);
  
  glcd_command(LCD_DISP_OFF,LCD_CHIP_ALL);
  glcd_rect_fill(0,0,LCD_RIGHT,LCD_BOTTOM,LCD_DOT_XOR);
  _delay_ms(400);
  
  glcd_command(LCD_DISP_ON,LCD_CHIP_ALL);
  _delay_ms(500);

  _delay_ms(DELAY);
}
static void demo_welcome_image(void)
{
	// manmoolsang welcome image
  glcd_clear();
  glcd_bitmap(manmool200x32, 0,0,LCD_DOT_SET);
  _delay_ms(DELAY);
}

static void demo_bmp(void)
{
  uint8_t i;

  glcd_clear();
 
  glcd_selectfont(Arial_Bold_14, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(46, 2); glcd_puts("Bitmap");
  glcd_gotoxy(46,16); glcd_puts("IMAGE");
  glcd_bitmap(android32x32, 0,0,LCD_DOT_SET);
  _delay_ms(DELAY);
 

  glcd_clear();
  glcd_bitmap(winstar200x32, 0,0,LCD_DOT_SET);
  _delay_ms(DELAY);
  
  glcd_clear();
  glcd_bitmap(apple32x32,     0, 0,LCD_DOT_SET);
  _delay_ms(600);
  
  glcd_bitmap(android32x32,  32, 0,LCD_DOT_SET);
  _delay_ms(600);
  
  glcd_bitmap(wolftrack32x32,64, 0,LCD_DOT_SET);
  _delay_ms(600);
 
  glcd_bitmap(font32x32     ,98, 0,LCD_DOT_SET);
   _delay_ms(600);

  glcd_bitmap(ArduinoIcon64x32, 134, 0,LCD_DOT_SET);
  _delay_ms(2000);

  glcd_rect_fill(31,0,63,LCD_BOTTOM,LCD_DOT_XOR);
  _delay_ms(1000);
  glcd_rect_fill(96,0,131,LCD_BOTTOM,LCD_DOT_XOR);
  _delay_ms(DELAY);
  
  glcd_clear();
  glcd_bitmap(atmel_old96x32,0,0, LCD_DOT_SET);
  glcd_bitmap(atmel96x32    ,104,0, LCD_DOT_SET);
  _delay_ms(DELAY);

  glcd_clear();                                                
  glcd_bitmap(forward_icon56x32, 1,0,LCD_DOT_SET);             
  glcd_bitmap(skype80x32, LCD_CHIP2_START_X,0,LCD_DOT_SET);    
  glcd_bitmap(back_icon56x32, LCD_CHIP3_START_X+4,0,LCD_DOT_SET);
  _delay_ms(DELAY);                                            

  for(i=0;i < LCD_BOTTOM;i++)
  {
   glcd_command(LCD_START_LINE + i, LCD_CHIP_1);
   glcd_command(LCD_START_LINE + LCD_BOTTOM -i, LCD_CHIP_2);
   glcd_command(LCD_START_LINE + i, LCD_CHIP_3);
   _delay_ms(130);
  }
  glcd_command(LCD_START_LINE + 0, LCD_CHIP_ALL);
  _delay_ms(DELAY);

  glcd_rect_fill(0,0,LCD_RIGHT,LCD_BOTTOM,LCD_DOT_XOR);
  for(i=1;i < LCD_BOTTOM;i++)
  {
   glcd_command(LCD_START_LINE + LCD_BOTTOM -i, LCD_CHIP_1);
   glcd_command(LCD_START_LINE + i, LCD_CHIP_2);
   glcd_command(LCD_START_LINE + LCD_BOTTOM -i, LCD_CHIP_3);
   _delay_ms(130);
  }
  glcd_command(LCD_START_LINE + 0, LCD_CHIP_ALL);
  _delay_ms(DELAY);

  glcd_clear();  
  glcd_bitmap(manmool200x32, 0,0,LCD_DOT_SET);
  glcd_rect_fill(0,0,LCD_RIGHT,LCD_BOTTOM,LCD_DOT_XOR);
  _delay_ms(DELAY);
} 
  
static void demo_line_graphics(void)
{
  uint8_t i,x,style;
  
  glcd_clear();
  glcd_cube(0,0,22,19, 10,10,32,29,LCD_DOT_SET,LCD_LINE_SOLID);
  
  glcd_selectfont(Arial_Bold_14, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(38, 2); glcd_puts("LINE Graphics");
  _delay_ms(DELAY);  
  
  glcd_clear();
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(1,0);  glcd_puts("Solid and Dotted");
  
  // horizontal line
  for(i=7;i<=LCD_BOTTOM;i+=12)
  {
    glcd_hline(0, i, LCD_RIGHT-1, LCD_DOT_SET, LCD_LINE_SOLID);   
    glcd_hline(0, i+6, LCD_RIGHT-1, LCD_DOT_SET, LCD_LINE_DOTTED);   
    _delay_ms(500);  
  }
  
  // vertical line
  for(i=0;i<=LCD_RIGHT;i+=25)
  {
    glcd_vline(i   ,7,LCD_BOTTOM, LCD_DOT_SET, LCD_LINE_SOLID);  
    glcd_vline(i+25,7,LCD_BOTTOM, LCD_DOT_SET, LCD_LINE_DOTTED);   
    i+=24;
    _delay_ms(500);  
  }
  _delay_ms(DELAY);
  
  glcd_clear();
  style = LCD_LINE_SOLID;
  for(i=0;i<15;i+=5)
  {
    glcd_rect(i, i, LCD_CENTER_X-i-2, LCD_BOTTOM-i, LCD_DOT_SET, style); 
    style = (style == LCD_LINE_SOLID ? LCD_LINE_DOTTED: LCD_LINE_SOLID);
    glcd_rect_round(LCD_CENTER_X+i+2, i, LCD_RIGHT-i, LCD_BOTTOM-i, LCD_DOT_SET, style,3); 
    glcd_gotoxy(20,12); glcd_puts("Rectangle");
    _delay_ms(400);
  }
  _delay_ms(DELAY); 

  // draw a diagonal line
  glcd_clear();	// clear display
  glcd_gotoxy(0,24); glcd_puts("Diagonal Line");
  for(i=0;i<=LCD_BOTTOM;i+=3)
  {
    glcd_line(0,0,LCD_RIGHT,i,LCD_DOT_SET,LCD_LINE_SOLID);
    _delay_ms(400);
  }
  x=0;
  for(i=60;i--;)
  {
    glcd_rect_fill(6,7,22,22,LCD_DOT_CLR); // clear previous spinner position  
    drawSpinner(x++, 14,14);    // draw new spinner position
    _delay_ms(80);
  }

  // draw a circle
  glcd_clear();
  glcd_gotoxy(1,0); glcd_puts("Circle");
  for(x=0;x<15;x++)
  {
    for(i=0;i<15;i+=3)
    {
      glcd_circle(LCD_CENTER_X+x+6,LCD_CENTER_Y+x, i*3, LCD_DOT_SET);     	
      _delay_ms(80);
      glcd_circle(LCD_CENTER_X+x+6,LCD_CENTER_Y+x, i*3 , LCD_DOT_CLR);     	
    }
  }
  
  glcd_rect(LCD_CENTER_X-1, 0, LCD_RIGHT, LCD_BOTTOM, LCD_DOT_SET, style);
  glcd_hline(LCD_CENTER_X-1, LCD_CENTER_Y, LCD_RIGHT, LCD_DOT_SET, LCD_LINE_DOTTED);
  glcd_vline(LCD_CENTER_X+32, 0, LCD_BOTTOM, LCD_DOT_SET, LCD_LINE_DOTTED);
  glcd_vline(LCD_CENTER_X+66, 0, LCD_BOTTOM, LCD_DOT_SET, LCD_LINE_DOTTED);
  for(i=0;i<12;i++)
  {
    glcd_circle(LCD_CENTER_X+32,LCD_CENTER_Y, i*3 , LCD_DOT_SET);     	
    glcd_circle(LCD_CENTER_X+66,LCD_CENTER_Y, i*3 , LCD_DOT_SET);     	
    _delay_ms(300);
  }
  _delay_ms(DELAY);
}

static void demo_font()
{
  glcd_clear();
  glcd_selectfont(Arial_Bold_14, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(44,10); glcd_puts("FONTs");
  glcd_bitmap(font32x32, 0,0,LCD_DOT_SET);
  _delay_ms(DELAY);
  
  // SystemFont5x7
  glcd_clear();
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,0); glcd_puts((char *)Lorem_Ipsum_Text);
  
  glcd_rect_fill(56,11,140,19,LCD_DOT_CLR);
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_XOR);
  glcd_gotoxy(56,12); glcd_puts(" SystemFont5x7 ");
  _delay_ms(DELAY);
  
  // Arial14
  glcd_clear();
  glcd_selectfont(Arial14, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,0); glcd_puts("THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG");
  
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_XOR);
  glcd_gotoxy(140,22); glcd_puts(" Arial14 ");
  _delay_ms(DELAY);
  
  // Arial14 Bold
  glcd_clear();
  glcd_selectfont(Arial_Bold_14, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,0); glcd_puts("The quick brown fox jumps over the lazy dog");
  
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_XOR);
  glcd_gotoxy(100,22); glcd_puts(" Arial_Bold_14 ");
  _delay_ms(DELAY);
  
  // Corsiva_12
  glcd_clear();
  glcd_selectfont(Corsiva_12, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,0); glcd_puts((char *)Lorem_Ipsum_Text);
  
  glcd_rect_fill(128,21,200,32,LCD_DOT_CLR);
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_XOR);
  glcd_gotoxy(130,22); glcd_puts(" Corsiva_12 ");
  _delay_ms(DELAY);
  
  // numbers
  glcd_clear();
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,4);  glcd_puts("Verdana12");                                
  glcd_gotoxy(0,20); glcd_puts("Verdana15");                                 
  
  glcd_selectfont(Verdana_digits_12, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(60,2); glcd_puts("+-/*0,123,456,789");

  glcd_selectfont(Verdana_digits_15, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(60,16); glcd_puts("+-/0,123,456,789");
 _delay_ms(DELAY);

  glcd_clear();
  glcd_selectfont(Verdana_digits_26, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,0); glcd_puts("010-0325-3253");
  
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,24);  glcd_puts("Verdana26");                                
 _delay_ms(DELAY);

  glcd_clear();
  glcd_selectfont(Verdana_digits_30, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(20,0); glcd_puts("2013-01-27");
  
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0,24);  glcd_puts("Verdana30");                                
 _delay_ms(DELAY);

 
}

static void demo_end()
{
	glcd_clear();
	
	demo_welcome_image();
	glcd_clear();	
  glcd_selectfont(Corsiva_12, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(34,10); glcd_puts("Thank You For Watching!");
	_delay_ms(DELAY);
	glcd_clear();
	_delay_ms(DELAY);
} 

ISR(TIMER1_OVF_vect)
{
	TCNT1 = 177;	          // 8bit timer initialize (1mili second)
	ISR_milisecond++;	      // 1 mili second
}

static void timer1_init(void)
{
	ISR_milisecond = 0;
	TCCR1B = _BV(CS12);   	// timer0 prescale (256)
	TCNT1  =  177;				  // 1milisecond overflow (256)
	TIMSK1 = _BV(TOIE1);		// timer0 interrupt enable
	sei();

	// 20000000Hz / 256 prescaler = 78125Hz
	// 1 / 78125Hz = TCNT0 Increase elaspse time is 0.0000128 Second
	// 0.001 Second / 0.0000128 Second = 78(78.125)  TCNT0 reach 1milisecond
	// TCNT0 Value is 255 - 78 = 177 (1milisend overflow value)

}
static void drawSpinner(uint8_t pos, uint8_t x, uint8_t y) 
{   
  // this draws an object that appears to spin
  switch(pos % 8) 
  {
    case 0 : glcd_line( x, y-8, x, y+8,    LCD_DOT_SET,LCD_LINE_SOLID);    break;
    case 1 : glcd_line( x+3, y-7, x-3, y+7,LCD_DOT_SET,LCD_LINE_SOLID);    break;
    case 2 : glcd_line( x+6, y-6, x-6, y+6,LCD_DOT_SET,LCD_LINE_SOLID);    break;
    case 3 : glcd_line( x+7, y-3, x-7, y+3,LCD_DOT_SET,LCD_LINE_SOLID);    break;
    case 4 : glcd_line( x+8, y, x-8, y,    LCD_DOT_SET,LCD_LINE_SOLID);    break;
    case 5 : glcd_line( x+7, y+3, x-7, y-3,LCD_DOT_SET,LCD_LINE_SOLID);    break;
    case 6 : glcd_line( x+6, y+6, x-6, y-6,LCD_DOT_SET,LCD_LINE_SOLID);    break; 
    case 7 : glcd_line( x+3, y+7, x-3, y-7,LCD_DOT_SET,LCD_LINE_SOLID);    break;
  } 
}

static void demo_performace(void)
{
  uint32_t before;
  uint32_t after;
  char str[10];
  
  timer1_init();
  
  glcd_clear();
  
  glcd_selectfont(SystemFont5x7, glcd_readfont, LCD_DOT_SET);
  glcd_gotoxy(0, 10); glcd_puts("START");
  
  before=ISR_milisecond;

  // 32 x 202 dot pixcel clear measure the time (msec)
  glcd_clear();
     
  after = ISR_milisecond - before;
  cli();
  
  glcd_puts_ltoa(after,10,str);
  
  glcd_gotoxy(0, 20); glcd_puts("FINISH:    msec");
  glcd_gotoxy(46,20); glcd_puts(str);
}