/*
 * $Id: sbn166g.c 2013/01/27 21:38:15 ssk  $
 *
 * This program is part of Winstar WG20232A-TMI-V#A with 202x32 Graphic LCD Tutorial.
 * Designed for AVANT SBN166G Dot-matrix STN LCD Driver with 32-row x 80-column
 * based LCDs on Atmels AVR MCUs
 *
 * Copyright (C) 2012  ssk.
 *
 * Originally Adapted from
 *    Most Functions   : GLCD Version 3 (Beta) Jun 9 2010 <http://arduino.cc/forum/index.php/topic,22208.0.html>
 *    Font Function&Data: Martin Thomas, Kaiserslautern, Germany <eversmith@heizung-thomas.de> 
 *                       avr-gcc 3.4.3/avr-libc 1.2.3 (WinAVR 2/2005)
 *    Cube Drawing     : Osama's Lab KS0108 GLCD library <https://sites.google.com/site/osamaslab>
 *    Line Drawing     : http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
 *    Circle Drawing   : http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
 *
 *
 * Changed & added additional constants for gcc version 4.3.3 (WinAVR 20100110), improved speed and optimized code
 * You need to be selective about the information you receive instead of accepting it unconditionally.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
 
/** 
 *
 * A Winstar WG20232A-TMI-V#A with 202x32 Graphic LCD module has been used
 * for testing. This module supports "read/write". 
 * There is option to read data from the SBN166G RAM. 
 *
 * IMPORTANT NOTE 
 *  
 * RESET Pin      
 * 
 * Hardware RESET and interface type selection.                                                                    
 * This pin is a dual function pin. It can be used to reset the SBN1661G_X and select the type of interface timing.
 * The hardware RESET is edge-sensitive. IT IS NOT LEVEL-SENSITIVE                                                 
 * 
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "sbn166g.h"
#include "delayNS.h"


// Low level LCD Controller Interface Support function
static uint8_t _glcd_read_data(void);
static void _glcd_dot(const uint8_t x, const uint8_t y, const uint8_t color);
static void _chip_select(uint8_t x);

/* Specify a controller between  0 and LCD_RIGHT  */
static void _chip_select(uint8_t x) 
{
	if( x < LCD_CHIP2_START_X)                          // 0 ~ LCD_CHIP2_START_X - 1 (Controller 1 area)
  	LCDCTRL_PORT |= _BV(LCD_CS1_PIN);   
  else 
  if(x >= LCD_CHIP2_START_X && x < LCD_CHIP3_START_X) // LCD_CHIP2_START_X ~ LCD_CHIP3_START_X - 1 (Controller 2 area)
    LCDCTRL_PORT |= _BV(LCD_CS2_PIN);   
  else 
  if( x >=  LCD_CHIP3_START_X)                        // LCD_CHIP3_START_X ~ LCD_RIGHT Controller 3 area)
    LCDCTRL_PORT |= _BV(LCD_CS3_PIN); 
}	

// routine to initialize the operation of the LCD display subsystem
void glcd_init(void)
{
  _delay_ms(250);

 	LCDDATA_DDR  = 0xff;  // all output     
  LCDCTRL_DDR  |= _BV(LCD_A0_PIN)  | _BV(LCD_RW_PIN)  | 
                  _BV(LCD_CS1_PIN) | _BV(LCD_CS2_PIN) | _BV(LCD_CS3_PIN) | 
                  _BV(LCD_RST_PIN);
                 
  LCDCTRL_PORT &= ~(_BV(LCD_CS1_PIN) | _BV(LCD_CS2_PIN) | _BV(LCD_CS3_PIN));  // deselect all chips (optional)
 
  // The hardware RESET is edge-sensitive. It is not level-sensitive. !!!
  // The value is relative to the RESET pulse edge. That is, 10 S after the last RESET edge, the device is completely reset
 	LCDCTRL_PORT  &= ~(_BV(LCD_RST_PIN));
 	_delay_us(50); 
 	
 	// The hardware RESET is edge-sensitive. It is not level-sensitive. !!!
 	LCDCTRL_PORT  |=   _BV(LCD_RST_PIN);    // 68-type microcontroller for interface.
    
  glcd_command(LCD_RESET, LCD_CHIP_ALL);
  _delay_ms(5);
  
  glcd_command(LCD_STATIC_OFF, LCD_CHIP_ALL);
  glcd_command(LCD_DUTY_32, LCD_CHIP_ALL);
  glcd_command(LCD_SET_ADC_NOR, LCD_CHIP_ALL);
  glcd_command(LCD_SET_RMW_END, LCD_CHIP_ALL);
  glcd_command(LCD_START_LINE, LCD_CHIP_ALL);
  glcd_command(LCD_DISP_ON, LCD_CHIP_ALL);
}

/**
 * Send LCD controller instruction command
 * Input: instruction to send to LCD controller
 * Returns: none
*/
inline void glcd_command(const uint8_t command, uint8_t device)
{ 
  LCDDATA_DDR = 0xff;
  
  LCDCTRL_PORT &= ~(_BV(LCD_A0_PIN));   // Low : Display Control data, High : Display data
  LCDCTRL_PORT &= ~(_BV(LCD_RW_PIN));   // Low : Write Control signal, High : Read Control signal
      
  LCDDATA_PORT = command;
  _delay_ns(LCD_tAS);                   // tAS1,2:Address setup time with respect to R/W,C/S,C/D (ctrl line changes to E high) 
  
  if( device & LCD_CHIP_1 ) LCDCTRL_PORT |= _BV(LCD_CS1_PIN);
  if( device & LCD_CHIP_2 ) LCDCTRL_PORT |= _BV(LCD_CS2_PIN);
	if( device & LCD_CHIP_3 ) LCDCTRL_PORT |= _BV(LCD_CS3_PIN);  
  _delay_ns(LCD_tEWW);                  // Enable pulse width WRITE(minimum E hi pulse width) 
  
  _chip_unselect();
}
/**
 * Send data byte to LCD controller
 * Input: instruction to send to LCD controller
 * Returns: none
*/
inline void glcd_data( uint8_t data)
{
  uint8_t yoffset  = _glcd_coord.y % 8;;
  uint8_t savedata =  0;
 
  if(_glcd_coord.x == LCD_CHIP2_START_X)							// chip2 X-address = 0
  	glcd_command(LCD_SET_COL, LCD_CHIP_2);
  else
  if(_glcd_coord.x == LCD_CHIP3_START_X)							// chip2 X-address = 0
  	glcd_command(LCD_SET_COL, LCD_CHIP_3);

  if(yoffset) 
  {
    // first page    
    savedata = _glcd_read_data();
    
    LCDCTRL_PORT |=  (_BV(LCD_A0_PIN));  // High : Display data,        Low : Display Control data
    LCDCTRL_PORT &= ~(_BV(LCD_RW_PIN));  // Low : Write Control signal, High : Read Control signal
    _delay_ns(LCD_tAS);                  // tAS1,2:Address setup time with respect to R/W,C/S,C/D (ctrl line changes to E high)
  
    _chip_select(_glcd_coord.x);
    
    savedata |= (data << yoffset);  
    LCDDATA_PORT  =  savedata;					 // write data    

    _delay_ns(LCD_tEWW);                 // Enable pulse width WRITE(minimum E hi pulse width)
    _chip_unselect();
  
    //  second page
    if((_glcd_coord.y+8) > LCD_BOTTOM)
    {	
    	glcd_gotoxy(_glcd_coord.x+1, _glcd_coord.y);
    	return;
    }
    	
    glcd_gotoxy(_glcd_coord.x, _glcd_coord.y+8);
    savedata = _glcd_read_data();
    
    LCDCTRL_PORT |=  (_BV(LCD_A0_PIN));  // High : Display data,        Low : Display Control data
    LCDCTRL_PORT &= ~(_BV(LCD_RW_PIN));  // Low : Write Control signal, High : Read Control signal
    _delay_ns(LCD_tAS);                  // tAS1,2:Address setup time with respect to R/W,C/S,C/D (ctrl line changes to E high)
  
    _chip_select(_glcd_coord.x);
   
    savedata |= data >> (8-yoffset);
    LCDDATA_PORT  =  savedata;					 // write data    
    _delay_ns(LCD_tEWW);                 // Enable pulse width WRITE(minimum E hi pulse width)     
    
    _chip_unselect();
    
    glcd_gotoxy(_glcd_coord.x+1, _glcd_coord.y-8); 
  } 
  else 
  {
    LCDCTRL_PORT |=  (_BV(LCD_A0_PIN));  // High : Display data,        Low : Display Control data
    LCDCTRL_PORT &= ~(_BV(LCD_RW_PIN));  // Low : Write Control signal, High : Read Control signal

    LCDDATA_PORT  =  data;						  
    _delay_ns(LCD_tAS);                  // tAS1,2:Address setup time with respect to R/W,C/S,C/D (ctrl line changes to E high)  
    
    _chip_select(_glcd_coord.x);
    _delay_ns(LCD_tEWW);                 // Enable pulse width WRITE(minimum E hi pulse width)    
     
    _chip_unselect(); 	
 		_glcd_coord.x++;
  }
}

 /**
 * Read a data byte from the given position
 *
 * @param x X Column byte location, a value from 0 to LCD_RIGHT
 * @param y Y Page byte location, a value from 0 to LCD_BOTTOM
 *
 * @return Data read from NT7502 ram
 #
 * x and y are relative to the 0,0 origin of the display which
 * is the upper left corner.
 * Requests to set pixels outside the range of the display will be ignored.
 *
 * @note for Non-Framebuffer
 *
*/
inline static uint8_t _glcd_read_data(void)
{  
  uint8_t data;

  glcd_command(LCD_SET_RMW_START, LCD_CHIP_ALL);    // Read-Modify-Write Start

  LCDDATA_DDR = 0x00;                  
  LCDCTRL_PORT |=   _BV(LCD_A0_PIN);   // High : Display data,        Low : Display Control data
  LCDCTRL_PORT |=   _BV(LCD_RW_PIN);   // High : Read Control signal, Low : Write Control signal  
  _delay_ns(LCD_tAS);                  // tAS1,2:Address setup time with respect to R/W,C/S,C/D (ctrl line changes to E high)                                                                                                   

  _chip_select(_glcd_coord.x);         // dummy read
  _chip_unselect();

  _chip_select(_glcd_coord.x);
  _delay_ns(LCD_tACC);                 // tAS1,2:Address setup time with respect to R/W,C/S,C/D (ctrl line changes to E high)                                                                                                   
  
  data = LCDDATA_INPUT;                // Get data from LCD data

  _chip_unselect();

  //glcd_gotoxy(_glcd_coord.x, _glcd_coord.y);
  glcd_command(LCD_SET_RMW_END, LCD_CHIP_ALL);     // Read-Modify-Write END

  LCDDATA_DDR = 0xff; 
	return(data);
}


 /**
 * go to at x,y to the given position
 *
 * @param x X coordinate, a value from 0 to LCD_RIGHT
 * @param y Y coordinate, a value from 0 to LCD_BOTTOM
 *
 * Sets the current position at location the specified x,y
 * x and y are relative to the 0,0 origin of the display which
 * is the upper left corner.
 * Requests to set pixels outside the range of the display will be ignored.
 *
 * @note for Non-Framebuffer
 *
*/
inline void glcd_gotoxy( uint8_t x,  uint8_t y) 
{
 	uint8_t device = 0;
 	
 	if(x > LCD_RIGHT || y > LCD_BOTTOM) return;
	
	_glcd_coord.x = x;								// save new coordinates
	_glcd_coord.y = y;
		
  y >>= 3;     // page coordinate of the cursor (0 ~ LCD_Y_BYTES)
	
  if(x < LCD_CHIP2_START_X)                          // 0 ~ LCD_CHIP2_START_X - 1 (Controller 1 area)
  {	
  	device = LCD_CHIP_1;
  }
  else 
  if(x >= LCD_CHIP2_START_X && x < LCD_CHIP3_START_X) // LCD_CHIP2_START_X ~ LCD_CHIP3_START_X - 1 (Controller 2 area)
  {
    device = LCD_CHIP_2;
    x -= LCD_CHIP2_START_X;
  }
  else 
  if( x >=  LCD_CHIP3_START_X)                        // LCD_CHIP3_START_X ~ LCD_RIGHT Controller 3 area)
  {
    device = LCD_CHIP_3;
    x -= LCD_CHIP3_START_X;
  }

	glcd_command(LCD_SET_COL  + x, device);
 	glcd_command(LCD_SET_PAGE + y, LCD_CHIP_ALL);
}
/**
 * set pixel at x,y to the given color
 *
 * @param x X coordinate, a value from 0 to LCD_RIGHT
 * @param y Y coordinate, a value from 0 to LCD_BOTTOM
 * @param color BLACK or CLEAR or INVERT
 *
 * Sets the pixel at location x,y to the specified color.
 * x and y are relative to the 0,0 origin of the display which
 * is the upper left corner.
 * Requests to set pixels outside the range of the display will be ignored.
 *
 * @note 
 *
*/
void _glcd_dot(const uint8_t x, const uint8_t y, const uint8_t color)
{ 
  if ( ( x > LCD_RIGHT ) || ( y > LCD_BOTTOM ) ) return;

	uint8_t data,rest;
	rest = y % 8;
	
	glcd_gotoxy(x, y - rest);			
	data = _glcd_read_data();	    // read data from display memory     
	switch (color) 
  { 
  	 case LCD_DOT_SET:
       data |=   _BV(rest);     // 0x01 << (rest);
       break;
    case LCD_DOT_CLR:
       data &= ~(_BV(rest));    // 0x01 << (rest));
       break;
    case LCD_DOT_XOR:
       data ^=  (_BV(rest));    // 0x01 << (rest));
       break;
    default: break;
  }       
  glcd_data(data);
}

/*
 * Draw line code is based on Bresenham's line algorithm
 * http://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm
 *
 * This routine replaces the Teiele's DrawLine() routine which was
 * much larger and much slower.
 *	-- bperrybap
*/
 
/*
 * First, define a few macros to make the DrawLine code below read more like
 * the wikipedia example code.
*/

#define _absDiff(x,y) ((x>y) ?  (x-y) : (y-x))
#define _swap(a,b) \
do\
{\
uint8_t t;\
	t=a;\
	a=b;\
	b=t;\
} while(0)
/**
 * Draw a Line
 *
 * @param x a value from 0 to LCD_RIGHT   (LCD_X_BYTES - 1)     indicating start x coordinate               
 * @param y a value from 0 to LCD_BOTTOM ((LCD_Y_BYTES * 8) -1) indicating start y coordinate               
 * @param height a value from 0 to LCD_RIGHT-y
 * @param color color of line
 * @param style style of line 
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * The line drawn will be height+1 pixels.
 *
 * @note 
 *
*/
void glcd_line(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, const uint8_t color, const uint8_t style)
{
  uint8_t deltax, deltay, x,y, steep;
  int8_t error, ystep;

  steep = _absDiff(y1,y2) > _absDiff(x1,x2);  

	if ( steep )
	{
		_swap(x1, y1);
		_swap(x2, y2);
	}

	if (x1 > x2)
	{
		_swap(x1, x2);
		_swap(y1, y2);
	}

	deltax = x2 - x1;
	deltay =_absDiff(y2,y1);  
	error = deltax / 2;
	y = y1;
	if(y1 < y2) ystep = 1;  else ystep = -1;

	for(x = x1; x <= x2; x++)
	{
		if(steep) 
			_glcd_dot(y,x, color); 
		else 
			_glcd_dot(x,y, color);
   		
   	error = error - deltay;
		if(error < 0)
		{
			y = y + ystep;
			error = error + deltax;
   	}
	}
}
	
/**
 * Draw a Horizontal Line
 *
 * @param x1 a value from 0 to LCD_RIGHT   indicating start x coordinate               
 * @param y1 a value from 0 to LCD_BOTTOM  indicating start y coordinate               
 * @param x2 a value from 0 to LCD_RIGHT   indicating end   x coordinate
 * @param color color of line
 * @param style style of line 
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * The line drawn will be width+1 pixels.
 *
 * @note 
 *
*/
void glcd_hline(uint8_t x1, const uint8_t y1, uint8_t x2, const uint8_t color, const uint8_t style)
{
   uint8_t x;
   
   for(x= x1; x<=x2; x += (style+1)) _glcd_dot(x,y1,color);
   
}

/**
 * Draw a Vertical Line
 *
 * @param x1 a value from 0 to LCD_RIGHT   indicating start x coordinate               
 * @param y1 a value from 0 to LCD_BOTTOM  indicating start y coordinate               
 * @param y2 a value from 0 to LCD_BOTTOM  indicating end   y coordinate
 * @param color color of line
 * @param style style of line 
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * The line drawn will be height+1 pixels.
 *
 * @note 
 *
*/
void glcd_vline(const uint8_t x1, const uint8_t y1, const uint8_t y2, const uint8_t color, const uint8_t style)
{
   uint8_t y;

   for(y= y1; y <= y2; y += (style+1)) _glcd_dot(x1, y, color);
   
   if(style == LCD_LINE_DOTTED) _glcd_dot(x1,y2,color);
}

/**
 * Draw a rectangle of given width and height
 *
 * @param x1 X coordinate of upper  left corner of the rectangle
 * @param y1 Y coordinate of upper  left corner of the rectangle
 * @param x2 X coordinate of lower right corner of the rectangle
 * @param y2 Y coordinate of lower right corner of the rectangle
 * @param color color of line
 * @param style style of line 
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * Draws a rectangle with the specified width and height.
 * The upper left corner at x,y and the lower right
 * corner at x+width,y+width.
 *
 * The length of the horizontal sides will be width+1 pixels
 * The length of the vertical sides will be height+1 pixels
 *
 * @note 
 *
*/
void glcd_rect(const uint8_t x1, const uint8_t y1, const uint8_t x2, const uint8_t y2, 
                   const uint8_t color, const uint8_t style)
{
  glcd_hline(x1, y1, x2, color, style);	// top     
  glcd_hline(x1, y2, x2, color, style);	// bottom  
  glcd_vline(x1, y1, y2, color, style);	// left    
  glcd_vline(x2, y1, y2, color, style);	// right   
} 
/**
 * Draw a rectangle with rounder corners
 *
 * @param x1 X coordinate of upper  left corner of the rectangle
 * @param y1 Y coordinate of upper  left corner of the rectangle
 * @param x2 X coordinate of lower right corner of the rectangle
 * @param y2 Y coordinate of lower right corner of the rectangle
 * @param color color of line
 * @param style style of line 
 * @param radius
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * Draws a rectangle the same as glcd_rect but with rounded corners.
 * Radius is a value from 1 to half the height or width of the rectangle.
 *
 * @note 
 *
*/
void glcd_rect_round(const uint8_t x1, const uint8_t y1, const uint8_t x2, const uint8_t y2, 
                         const uint8_t color, const uint8_t style, const uint8_t radius) 
{
	int16_t tSwitch; 
	uint8_t xoffset, yoffset;
	
	if(radius<1) return;
	
	xoffset = 0;
	yoffset = radius;
 	tSwitch = 3 - 2 * radius;
	
	while(xoffset <= yoffset)
	{
    _glcd_dot(x1+radius - xoffset, y1+radius - yoffset, color);
    _glcd_dot(x1+radius - yoffset, y1+radius - xoffset, color);
  
    _glcd_dot(x2-radius + xoffset, y1+radius - yoffset, color);
    _glcd_dot(x2-radius + yoffset, y1+radius - xoffset, color);

    _glcd_dot(x2-radius + xoffset, y2-radius + yoffset, color);
    _glcd_dot(x2-radius + yoffset, y2-radius + xoffset, color);
  
    _glcd_dot(x1+radius - xoffset, y2-radius + yoffset, color);
    _glcd_dot(x1+radius - yoffset, y2-radius + xoffset, color);

	  if(tSwitch <= 0) 
	  {
	  	tSwitch += (4 * xoffset + 6);
	  } 
	  else 
	  {
	  	tSwitch += (4 * (xoffset - yoffset) + 10);
	  	yoffset--;
//	  	yoffset -= (style+1);
	  }
	  xoffset++;
//	  xoffset += (style+1);
	}
	
  glcd_hline(x1+ radius+1, y1, x2-radius-1, color, style);	// top     
  glcd_hline(x1+ radius+1, y2, x2-radius-1, color, style);	// bottom  
  glcd_vline(x1,y1+radius,    y2-radius, color, style);	// left    
  glcd_vline(x2,y1+radius,    y2-radius, color, style);	// right
}
/**
 * Draw a rectangle with fill
 *
 * @param x1 X coordinate of upper  left corner of the rectangle
 * @param y1 Y coordinate of upper  left corner of the rectangle
 * @param x2 X coordinate of lower right corner of the rectangle
 * @param y2 Y coordinate of lower right corner of the rectangle
 * @param color color of line
 * @param style style of line 
 * @param radius
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 *
 * Draws a rectangle the same as glcd_rect but with fill area.
 *
 * @note 
 *
*/
void glcd_rect_fill(const uint8_t x1, const uint8_t y1, uint8_t x2, const uint8_t y2, const uint8_t color)
{
   uint8_t i;
   for (i=y1; i<=y2; i++) glcd_hline(x1, i, x2, color, LCD_LINE_SOLID);
}

/**
 * Draw a Circle
 *
 * @param xCenter X coordinate of the center of the circle
 * @param yCenter Y coordinate of the center of the circle
 * @param radius radius of circle
 * @param color
 * @param style
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * Draws a circle of the given radius extending out from
 * the center pixel.
 * The circle will fit inside a rectanglular area bounded by
 * x-radius,y-radius and x+radius,y+radius
 *
 * Because the circle is drawn from the center pixel out,
 * the diameter will be 2 * radius +1 pixels.
 *
 * @note 
 *
*/
void glcd_circle(const uint8_t xcenter, const uint8_t ycenter, 
                const uint8_t radius, const uint8_t color)
{
  glcd_rect_round(xcenter-radius, ycenter-radius, xcenter+radius, ycenter+radius, color, LCD_LINE_SOLID, radius);
}

/**
 * Draw a cuboid
 *
 * @param x11 X coordinate of upper left point of the first surface  
 * @param y11 Y coordinate of upper left point of the first surface  
 * @param x21 X coordinate of lower right point of the first surface 
 * @param y21 Y coordinate of lower right point of the first surface 
 * @param x21 X coordinate of upper left point of the second surface 
 * @param y21 Y coordinate of upper left point of the second surface 
 * @param x22 X coordinate of lower right point of the second surface
 * @param y22 Y coordinate of lower right point of the second surface
 * @param color color of line
 * @param style style of line 
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * @note 
 *
*/
void glcd_cube(const uint8_t x11, const uint8_t y11, const uint8_t x12, const uint8_t y12, 
              const uint8_t x21, const uint8_t y21, const uint8_t x22, const uint8_t y22, 
              const uint8_t color, const uint8_t style)
{
  glcd_rect(x11,y11,x12,y12,color,style);
  glcd_rect(x21,y21,x22,y22,color,style);
  glcd_line(x11,y11,x21,y21,color,style);
  glcd_line(x12,y11,x22,y21,color,style);
  glcd_line(x11,y12,x21,y22,color,style);
  glcd_line(x12,y12,x22,y22,color,style);
}
/**
 * Draw a glcd bitmap image
 *
 * @param bitmap a ponter to the bitmap data
 * @param x the x coordinate of the upper left corner of the bitmap
 * @param y the y coordinate of the upper left corner of the bitmap
 * @param color BLACK, WHITE or INVERT
 *
 * Draws a bitmap image with the upper left corner at location x,y
 * The bitmap data is assumed to be in program memory.
 *
 * @note : Bit byte Aligment : LSB to MSB, Top to Bottom !
 */
void glcd_bitmap(uint8_t* bitmap, uint8_t x, uint8_t y,  const uint8_t color)
{
  uint8_t width, height;
  uint8_t i, j;
  uint8_t displayData;

  width  = pgm_read_byte(bitmap++); 
  height = pgm_read_byte(bitmap++);
  
  
  height >>= 3;    // height / 8

  for(j = 0; j < height; j++)
  {
    glcd_gotoxy(x, y + (j*8) );
	  for(i = 0; i < width; i++) 
	  {
		   displayData = pgm_read_byte(bitmap++);
		   
		   if(color == LCD_DOT_CLR) displayData &= ~displayData;
		   else 
		   	if(color == LCD_DOT_XOR) displayData ^= displayData;
		   		
       glcd_data(displayData);
	 }
  }
}
/*******************************************************************************
convert unsigned long int to characters in LCD
Input:    value unsigned long int number to be displayed
          radix  2(binary),
                10(decimal),
                16(hexadecimal)
Returns:  none
*******************************************************************************/
#define NUMBER_OF_DIGITS 32
void glcd_puts_ltoa(int32_t value, uint8_t radix, char *str)
{
  uint8_t  i = 0;
  uint8_t  index = NUMBER_OF_DIGITS;
  static char buffer[NUMBER_OF_DIGITS] = "0000000000000000000000000000000\0";    // buffer for NUMBER_OF_DIGITS + '\0'
 
  // Negative Value 
  if (value < 0 && radix == 10)
  {
  	i = 0 ;
    str[i] = '-';
    value *= -1;
    i++;
  }
  
  do
  {
    buffer[--index] = '0' + (value % radix);
    if ( buffer[index] > '9') buffer[index] += 'A' - ':';  // continue with A, B,...

    value /= radix;
  } while (value != 0);
  
  do
  {
  	str[i++] = buffer[index++];
  } 
  while ( index < NUMBER_OF_DIGITS );
  str[i] = '\0';
}

/**
 * EOF
*/